SystemOrganization addCategory: #Rest!
SystemOrganization addCategory: #'Rest-Tests'!
SystemOrganization addCategory: #'Rest-Handlers'!
SystemOrganization addCategory: #'Rest-Services'!

Object subclass: #RestHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Handlers'!

!RestHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 15:18'!
process: anHttpRequest
	"Handles an http request. Should answer with an http response."
	
	self subclassResponsibility! !

RestHttpHandler subclass: #RestPerformHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Handlers'!

!RestPerformHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 15:25'!
process: anHttpRequest
	"Takes first element in path as classname and sends message from third path element to that class."

	|pc class selector result|
	pc := (URI fromString: anHttpRequest url) pathComponents.
	class := pc first asSymbol.
	selector := pc second asSymbol.
	
	Smalltalk at: class ifPresent: [:c | 
		(c class allSelectors includes: selector) ifTrue: [
			result := c perform: selector]].

	^ HttpResponse fromString: result printString.! !

RestHttpHandler subclass: #RestResourceHttpHandler
	instanceVariableNames: 'resources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Handlers'!

!RestResourceHttpHandler methodsFor: 'http-post-helper' stamp: 'mt 2/16/2009 17:18'!
create: resClass from: fields
	"Works only with a new project or a new user."
	
	| entity |
	entity := resClass new.
	fields keysAndValuesDo: [:key :value |
		entity instVarNamed: key asString put: value first].
	resClass perform: ('add', (self resolveName: resClass) capitalized, ':') asSymbol with: entity.
	^ HttpResponse status: #ok contents: entity id asString! !

!RestResourceHttpHandler methodsFor: 'http-delete-helper' stamp: 'mt 2/16/2009 17:36'!
delete: resClass id: id
	"Delete a resource with the specified identifier. Works only with projects and users."

	| entity |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	resClass
		perform: ('delete', (self resolveName: resClass) capitalized, ':') asSymbol
		with: entity.
	^ HttpResponse status: #ok contents: 'deleted'.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 16:35'!
getRequestChainFor: aPathComponentList
	"Create a list of associations for the request. That list contains resource classes and their requested ids."
	
	| chain resClass idNumber |
	chain := OrderedCollection new.
	aPathComponentList size odd ifTrue: [aPathComponentList add: 'nil'].
	aPathComponentList pairsDo: [:resName :idName | 
		resClass := self resolveResource: resName ifAbsent: [nil].
		idName = 'nil'
			ifTrue: [idNumber := nil]
			ifFalse: [idNumber := idName asNumber].
		resClass ifNotNil: [chain add: (resClass -> idNumber)]].
	^ chain! !

!RestResourceHttpHandler methodsFor: 'initialization' stamp: 'mt 2/16/2009 15:41'!
initialize
	"Register all accessible resources."
	
	super initialize.
	resources := Dictionary new.
	self registerXpfEntityClasses.! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 17:37'!
process: anHttpRequest 
	"Create a resource chain from the path of the http request. Fail if that chain is empty. Redirect processing according to used verb (POST, PUT, GET, DELETE)."

	| resChain |
	resChain := self getRequestChainFor: (URI fromString: anHttpRequest url) pathComponents.
	resChain isEmpty ifTrue: [^ self responseForFailure: anHttpRequest].
	
	anHttpRequest isPostRequest   ifTrue: [^ self processPost:   resChain fields: anHttpRequest postFields].
	anHttpRequest isGetRequest    ifTrue: [^ self processGet:    resChain].
	anHttpRequest isPutRequest    ifTrue: [^ self processPut:    resChain fields: anHttpRequest postFields].
	anHttpRequest isDeleteRequest ifTrue: [^ self processDelete: resChain].! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:59'!
processDelete: aResourceChain 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	^ self delete: resClass id: id! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:57'!
processGet: aResourceChain 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	
	id isNil
		ifTrue: [^ self read: resClass]
		ifFalse:[^ self read: resClass id: id].! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:56'!
processPost: aResourceChain fields: someFields 
	
	|resClass|
	resClass := aResourceChain first key.
	^ self create: resClass from: someFields! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:58'!
processPut: aResourceChain fields: someFields 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	^ self update: resClass from: someFields id: id! !

!RestResourceHttpHandler methodsFor: 'http-get-helper' stamp: 'mt 2/16/2009 17:24'!
read: resClass
	"Read information about all entities of a given class. Works only for projects or users."
	
	| response |
	response := resClass all collect: [:entity|
		Dictionary newFrom: (
			resClass allInstVarNames collect: [:name| name -> (entity instVarNamed: name)])].
	^ HttpResponse status: #ok contents: (Json render: response).! !

!RestResourceHttpHandler methodsFor: 'http-get-helper' stamp: 'mt 2/16/2009 17:36'!
read: resClass id: id
	"Read information about one special resource with the given id. Works only for projects and users."

	| entity response |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	response := Dictionary new.
	XpfProject allInstVarNames do:
		[:name| response at: name put: (entity instVarNamed: name)].
	^ HttpResponse status: #ok contents: (Json render: response).! !

!RestResourceHttpHandler methodsFor: 'initialization' stamp: 'mt 2/16/2009 15:42'!
registerXpfEntityClasses
	"Add all subclasses of XpfEntity to the pool of accessible resources. Example: 'XpfUserStory' will have the key 'userstory' to lookup the entity class."

	XpfEntity allSubclasses do: [:aClass |
		resources
			at: (aClass name copyReplaceAll: 'Xpf' with: '') asLowercase
			put: aClass]! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 17:35'!
resolveName: aClass
	"Inverted lookup in the pool of registered resourcen."

	resources keysAndValuesDo: [:key :value |
		value = aClass ifTrue: [^ key]].
	^ nil.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 16:45'!
resolveResource: aResourceName ifAbsent: aBlock
	"Try to resolve the given resource name to a key in the registered resource pool. Plural will be converted to singular: userstories -> userstory, projects -> project. The resource name has to be lowercase."

	| resourceKey |
	resourceKey := aResourceName.
	(resourceKey endsWith: 'ies')
		ifTrue:[resourceKey := (resourceKey truncateTo: resourceKey size - 3), 'y'].
	(resourceKey endsWith: 's')
		ifTrue:[resourceKey := resourceKey truncateTo: resourceKey size - 1].
		
	^ resources at: resourceKey ifAbsent: aBlock.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 17:37'!
responseForFailure: anHttpRequest 
	^ nil asHttpResponseTo: anHttpRequest! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 17:35'!
responseForNotFound: resource id: id
	^ HttpResponse
		status: #notFound
		contents:
			(self resolveName: resource capitalized),
			' with id ', id asString,
			' not found.'! !

!RestResourceHttpHandler methodsFor: 'http-put-helper' stamp: 'mt 2/16/2009 17:36'!
update: resClass from: fields id: id
	"Works only with projects and users."

	| entity |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	fields keysAndValuesDo: [:key :value |
		entity instVarNamed: key asString put: value first].
	^ HttpResponse status: #ok contents: 'updated'! !

Object subclass: #RestLogService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Services'!

!RestLogService commentStamp: 'mt 2/16/2009 14:53' prior: 0!
You can use me to log messages whenever you want. I will decide what to do with that messages. Normally they will be shown on the Transcript.!

!RestLogService class methodsFor: 'service' stamp: 'cs 2/7/2009 14:20'!
log: aCollection
	Transcript show: aCollection first asString; cr.! !

!HTTPSocket class methodsFor: '*rest-tests' stamp: 'cs 2/15/2009 15:47'!
httpDelete: t1
	| t5 |
	t5 := self
				httpDeleteDocument: t1
				args: nil
				accept: '*/*'
				request: ''.
	t5 isString
		ifTrue: [^ t5].
	^ (RWBinaryOrTextStream with: t5 content) reset! !

!HTTPSocket class methodsFor: '*rest-tests' stamp: 'cs 2/15/2009 15:44'!
httpDeleteDocument: t1 args: t2 accept: t3 request: t4 
	| t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16 t17 t18 t19 t20 |
	Socket initializeNetwork.
	t11 := (t1 asLowercase beginsWith: 'http://')
				ifTrue: [t1 copyFrom: 8 to: t1 size]
				ifFalse: [t1].
	t11 := t11 copyUpTo: $#.
	t5 := t11 copyUpTo: $/.
	t12 := t11 copyFrom: t5 size + 1 to: t11 size.
	(t5 includes: $:)
		ifTrue: [t16 := t5 indexOf: $:.
			t7 := (t5 copyFrom: t16 + 1 to: t5 size) asNumber.
			t5 := t5 copyFrom: 1 to: t16 - 1]
		ifFalse: [t7 := self defaultPort].
	t12 size = 0
		ifTrue: [t12 := '/'].
	t2
		ifNotNil: [t12 := t12
						, (self argString: t2)].
	(self shouldUseProxy: t5)
		ifTrue: [t12 := 'http://' , t5 , ':' , t7 printString , t12.
			t17 := self httpProxyServer.
			t18 := self httpProxyPort]
		ifFalse: [t17 := t5.
			t18 := t7].
	t6 := NetNameResolver addressForName: t17 timeout: 20.
	t6
		ifNil: [^ 'Could not resolve the server named: ' , t17].
	3
		timesRepeat: [t8 := HTTPSocket new.
			t8 connectTo: t6 port: t18.
			(t8
					waitForConnectionUntil: (self deadlineSecs: 30))
				ifFalse: [Socket deadServer: t17.
					t8 destroy.
					^ 'Server ' , t17 , ' is not responding'].
			t8 sendCommand: 'DELETE ' , t12 , ' HTTP/1.0' , CrLf
					, (t3
							ifNil: ['']
							ifNotNil: ['ACCEPT: ' , t3 , CrLf]) , 'ACCEPT: text/html' , CrLf , HTTPBlabEmail , t4 , self userAgentString , CrLf , 'Host: ' , t5 , ':' , t7 printString , CrLf.
			t13 := t8
						getResponseUpTo: CrLf , CrLf
						ignoring: (String with: CR).
			t9 := t13 at: 1.
			t14 := t13 at: 3.
			t9 isEmpty
				ifTrue: [t15 := 'server aborted early']
				ifFalse: [t8 header: t9.
					t10 := t8 getHeader: 'content-length'.
					t10
						ifNotNil: [t10 := t10 asNumber].
					t19 := t8 getHeader: 'content-type'.
					t8 responseCode first = $3
						ifTrue: [t20 := t8 getHeader: 'location'.
							t20
								ifNotNil: [Transcript show: 'redirecting to ' , t20;
										 cr.
									t8 destroy.
									t20 := self
												expandUrl: t20
												ip: t6
												port: t18.
									^ self
										httpGetDocument: t20
										args: t2
										accept: t3
										request: t4]].
					t15 := t8 getRestOfBuffer: t14 totalLength: t10.
					t8 responseCode first = $4
						ifTrue: [^ t9 , t15 contents].
					nil].
			t8 destroy.
			t15 class ~~ String
				ifTrue: [^ MIMEDocument
						contentType: t19
						content: t15 contents
						url: t1].
			t15 = 'server aborted early'
				ifTrue: [^ t15].
			nil].
	{'HTTPSocket class>>httpGetDocument:args:accept:request:'. t15. t1} inspect.
	^ 'some other bad thing happened!!'! !

!HTTPSocket class methodsFor: '*rest-tests' stamp: 'cs 2/15/2009 15:50'!
httpPut: t1 args: t2
	| t4 |
	t4 := self
				httpPutDocument: t1
				args: t2
				accept: '*/*'
				request: ''.
	t4 isString
		ifTrue: [^ t4].
	^ RWBinaryOrTextStream with: t4 content! !

!HTTPSocket class methodsFor: '*rest-tests' stamp: 'cs 2/16/2009 01:42'!
httpPutDocument: t1 args: t2 accept: t3 request: t4 
	| t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 |
	Socket initializeNetwork.
	t14 := Url absoluteFromText: t1.
	t8 := t14 fullPath.
	t15 := t2
				ifNil: ['']
				ifNotNil: [t15 := self argString: t2.
					t15 first = $?
						ifTrue: [t15 := t15 copyFrom: 2 to: t15 size]].
	t5 := HTTPSocket new.
	t5 := self
				initHTTPSocket: t14
				wait: (self deadlineSecs: 30)
				ifError: [:t16 | ^ t16].
	t5 sendCommand: 'PUT ' , t8 , ' HTTP/1.0' , CrLf
			, (t3
					ifNil: ['']
					ifNotNil: ['ACCEPT: ' , t3 , CrLf]) , 'ACCEPT: text/html' , CrLf , HTTPProxyCredentials , HTTPBlabEmail , t4 , self userAgentString , CrLf , 'Content-type: application/x-www-form-urlencoded' , CrLf , 'Content-length: ' , t15 size printString , CrLf , 'Host: ' , t14 authority , CrLf.
	t5 sendCommand: t15.
	t9 := t5
				getResponseUpTo: CrLf , CrLf
				ignoring: (String with: CR).
	t6 := t9 at: 1.
	t10 := t9 at: 3.
	t5 header: t6.
	t7 := t5 getHeader: 'content-length'.
	t7
		ifNotNil: [t7 := t7 asNumber].
	t12 := t5 getHeader: 'content-type'.
	t5 responseCode first = $3
		ifTrue: [t13 := t5 getHeader: 'location'.
			t13
				ifNotNil: [t13 := (t14 newFromRelativeText: t13) asString.
					self flag: #refactor.
					t5 destroy.
					^ self httpGetDocument: t13 accept: t3]].
	t11 := t5 getRestOfBuffer: t10 totalLength: t7.
	t5 responseCode = '401'
		ifTrue: [^ t6 , t11 contents].
	t5 destroy.
	^ MIMEDocument
		contentType: t12
		content: t11 contents
		url: t1! !

!XpfEntity methodsFor: '*rest' stamp: 'cs 2/16/2009 00:40'!
jsonWriteOn: aStream
	self id jsonWriteOn: aStream! !

!Collection methodsFor: '*rest' stamp: 'cs 2/16/2009 00:38'!
jsonWriteOn: aStream
	| needComma |
	needComma := false.
	aStream nextPut: $[.
	self do: [:v |
		needComma
			ifTrue: [ aStream nextPutAll: ', ' ]
			ifFalse: [ needComma := true ].
		v jsonWriteOn: aStream.
	].
	aStream nextPut: $].! !

!Dictionary methodsFor: '*rest' stamp: 'mt 2/16/2009 10:28'!
keysAndValuesCollect: aBlock 
	| newDict |
	newDict := Dictionary new.
	self keysAndValuesDo: [:key :value | newDict add: (aBlock value: key value: value)].
	^ newDict! !

HttpService subclass: #RestHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Services'!

RestHttpService class
	instanceVariableNames: 'service'!

!RestHttpService commentStamp: 'mt 2/16/2009 14:56' prior: 0!
I am a restful http service. I am primarily used for the XP integration platform project. (Package: 'Xpf')!

RestHttpService class
	instanceVariableNames: 'service'!

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 15:14'!
isRunning
	^service isNil not and: [service isRunning]! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 14:57'!
port
	^8082! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 15:10'!
service
	^service! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 15:04'!
start
	"Start exactly one http service on port 8082. This service will be closed first if you call this message twice."

	self stop.
	service := (RestHttpService on: self port)
		start;
		yourself.! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 14:52'!
stop
	"Shutdown the server. Does nothing if server is not running."

	service ifNotNil:
		[service unregister.
		service := nil].! !

!RestHttpService methodsFor: 'initialization' stamp: 'mt 2/2/2009 10:12'!
initialize
	super initialize.
	handlers := OrderedCollection new	"needed for priorities"! !

!RestHttpService methodsFor: 'serving' stamp: 'cs 2/15/2009 16:07'!
installHandler: aRequestHandlerClass on: aPathString
	(handlers anySatisfy: [:ass | ass key = aPathString ])
		ifTrue: [^ false].
	handlers add: aPathString -> aRequestHandlerClass! !

!RestHttpService methodsFor: 'serving' stamp: 'mt 2/16/2009 18:02'!
processHttpRequest: request
	
	| activeHandler path |
	request isPutRequest ifTrue: [Transcript show: request fields].
	activeHandler := (handlers detect: [:ass| 
		path := ass key.
		request url includesSubString: path
		]) value.
	^(activeHandler new) process: request! !

TestCase subclass: #RestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests'!

RestTest subclass: #RestHttpServiceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests'!

!RestHttpServiceTest methodsFor: 'as yet unclassified' stamp: 'mt 2/17/2009 16:22'!
setUp
! !

!RestHttpServiceTest methodsFor: 'as yet unclassified' stamp: 'mt 2/17/2009 16:22'!
testStartAndStop
	
	RestHttpService start.
	(Delay forMilliseconds: 100) wait.
	
	self should: (RestHttpService service isKindOf: RestHttpService).
	self should: RestHttpService isRunning.
	
	RestHttpService stop.
	(Delay forMilliseconds: 100) wait.
	
	self should: (RestHttpService service == nil).
	self shouldnt: RestHttpService isRunning.! !

RestTest subclass: #RestPerformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests'!

!RestPerformTest methodsFor: 'running' stamp: 'mt 2/16/2009 15:24'!
setUp
	super setUp.
	RestHttpService service 
		installHandler: RestPerformHttpHandler
		on: '/'! !

!RestPerformTest methodsFor: 'tests' stamp: 'mt 2/16/2009 15:24'!
testPort
	| response |
	response := self get: '/RestHttpService/port'.
	self should: response contents = RestHttpService port asString.! !

RestTest subclass: #RestResourceTest
	instanceVariableNames: 'testProject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests'!

!RestResourceTest methodsFor: 'running' stamp: 'mt 2/16/2009 15:25'!
setUp
	"Install a handler for http service. Add a test project to the database."
	
	super setUp.
	RestHttpService service 
		installHandler: RestResourceHttpHandler
		on: '/'.
	testProject := XpfProject addProject:
		(XpfProject new
			title: 'TestProjekt';
			yourself).! !

!RestResourceTest methodsFor: 'running' stamp: 'cs 2/16/2009 01:29'!
tearDown
	super tearDown.
	(XpfProject all includes: testProject)
		ifTrue: [XpfProject deleteProject: testProject]! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/16/2009 15:26'!
testCreateProject
	"Try to create a new project via http POST request. Verify this by looking into the database directly."

	| response projectCount newProject |
	projectCount := XpfProject all size.
	response := self
		post: '/projects/'
		fields: (Dictionary with: 'title' -> 'Test').
	self should: XpfProject all size = (projectCount + 1).
	
	newProject := XpfProject all detect:
		[:aProject | aProject id = response asNumber].
	self should: newProject title = 'Test'.! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/16/2009 15:27'!
testCreateUserStory
	"Try to create a new user story via http POST request. Verify this by looking into the database directly."
	
	| response storyCount newStory |
	self fail.
	storyCount := testProject userStories size.
	response := self
		post: '/userstories/'
		fields: (Dictionary
			with: 'title' -> 'Test'
			with: 'priority' -> 5
			with: 'timeEstimated' -> 42
			with: 'project' -> 23).
	self should: testProject userStories size = (storyCount + 1).
	newStory := testProject userStories last.
	self should: newStory id = response contents asNumber.
	self should: newStory title = 'Test'.
	self should: newStory priority = 5.
	self should: newStory timeEstimated = 42.
	self should: newStory project = testProject.! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/16/2009 15:27'!
testDeleteProject
	"Try to delete a project via http DELETE request. Verify this by looking into the database directly."

	| projectCount |
	projectCount := XpfProject all size.
	self delete: '/projects/', testProject id asString.
	self should: XpfProject all size = (projectCount - 1).
	self shouldnt:	(XpfProject all anySatisfy:
			[:aProject | aProject id = testProject id]).! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/16/2009 15:28'!
testReadProject
	"Try to get information about a project via http GET request. Verify this by looking into the database directly."

	| response |	
	response := self get: '/projects/', testProject id asString.
	response := Json readFrom: (ReadStream on: response).
	self should: testProject title = (response at: 'title').
	self should: testProject id = (response at: 'id').! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/16/2009 15:30'!
testReadProjects
	"Try to get a list with rest urls to all projects via http GET request. Verify this by looking into the database directly."

	| response responseProject |
	response := self get: '/projects/'.
	response := Json readFrom: (ReadStream on: response).
	self should: XpfProject all size = response size.
	
	responseProject := response detect: [:each| (each at: 'id') = testProject id].
	self should: testProject title = (responseProject at: 'title').
	self should: testProject id = (responseProject at: 'id').! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/16/2009 16:49'!
testRequestChain
	"Check whether request chains are created correctly."
	
	| pc1 pc2 pc3 rc1 rc2 rc3|
	pc1 := {'projects'. '23'. 'userstory'. '7'. 'task'. '12'} asOrderedCollection.
	pc2 := {'projects'. '23'. 'userstories'} asOrderedCollection.
	pc3 := {'this'. 'is'. 'invalid'} asOrderedCollection.
	
	rc1 := RestResourceHttpHandler new getRequestChainFor: pc1.
	rc2 := RestResourceHttpHandler new getRequestChainFor: pc2.
	rc3 := RestResourceHttpHandler new getRequestChainFor: pc3.
	
	self should: rc1 first = (XpfProject -> 23).
	self should: rc1 second = (XpfUserStory -> 7).
	self should: rc1 third = (XpfTask -> 12).
	
	self should: rc2 first = (XpfProject -> 23).
	self should: rc2 second = (XpfUserStory -> nil).
	
	self should: rc3 isEmpty.! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/16/2009 15:30'!
testUpdateProject
	"Try to update information for a project via http PUT request. Verify this by looking into the database directly."
	
	| projectCount |
	projectCount := XpfProject all size.
	self
		put: '/projects/', testProject id asString
		fields: (Dictionary with: 'title' -> 'NewTest').
	self should: XpfProject all size = projectCount.
	self should: testProject title = 'NewTest'.! !

!RestTest class methodsFor: 'testing' stamp: 'cs 2/15/2009 14:14'!
isAbstract
	^ self name = 'RestTest'! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/16/2009 10:41'!
delete: aPath
	"Send an HTTP request with DELETE to localhost at the port of the (singleton) rest HTTP service."
	
	^ (HTTPSocket httpDelete: ('http://localhost:{1}{2}' format: {RestHttpService port asString. aPath})) contents asString.! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/16/2009 10:39'!
get: aPath
	"Send an HTTP request with GET to localhost at the port of the (singleton) rest HTTP service."

	^ (HTTPSocket httpGet: ('http://localhost:{1}{2}' format: {RestHttpService port asString. aPath})) contents asString! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/16/2009 10:37'!
post: aPath fields: aDictionary
	"Send an HTTP request with POST to localhost at the port of the (singleton) rest HTTP service. Append the given fields to the request body."
	
	^ (HTTPSocket
		httpPost: ('http://localhost:{1}{2}' format: {RestHttpService port asString. aPath})
		args:	(aDictionary keysAndValuesCollect: [:key :value | key -> {value}])
		accept: '*/*'
	) contents asString! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/16/2009 10:34'!
put: aPath fields: aDictionary
	"Send an HTTP request with PUT to localhost at the port of the (singleton) rest HTTP service. Append the given fields to the request body."
	
	^ (HTTPSocket
		httpPut:	('http://localhost:{1}{2}' format: {RestHttpService port asString. aPath})
		args:	(aDictionary keysAndValuesCollect: [:key :value | key -> {value}])
	) contents asString! !

!RestTest methodsFor: 'running' stamp: 'mt 2/16/2009 15:05'!
setUp
	RestHttpService start.
	"wait until the service is up"
	(Delay forMilliseconds: 100) wait.! !

!RestTest methodsFor: 'running' stamp: 'cs 2/15/2009 14:27'!
tearDown
	RestHttpService stop.! !
